(*

   TFileIterator Component
   Version 01.00.00
   (C) 1996, Glen Why

   Version history:


*)
unit FileOperator;

interface
uses
 Windows, Classes, Forms, ShellAPI;

type

  TFileOperation = ( foCopy, foDelete, foMove, foRename );
  TFileOperationFlag = ( foAllowUndo, foConfirmMouse, foFilesOnly,
    foMultiDestFiles, foNoConfirmation, foNoConfirmMkDir,
     foRenameOnCollision, foSilent, foSimpleProgress );
  TFileOperationFlags = set of TFileOperationFlag;

  TFileOperator = class( TComponent )
  private
    FData :TShFileOpStruct;
    FFrom :String;
    FTo :String;
    FProgressTitle :String;
    FParentForm :TForm;
    procedure SetOperation( Value :TFileOperation );
    function GetOperation :TFileOperation;
    procedure SetFrom( const Value :String );
    procedure SetTo( const Value :String );
    function GetWantMappingHandle :Boolean;
    procedure SetWantMappingHandle ( Value :Boolean );
    procedure SetFlags( Value :TFileOperationFlags );
    function GetFlags :TFileOperationFlags;
    function GetOperFlag( F :Cardinal ) :Boolean;
    procedure SetOperFlag( F :Cardinal; V :Boolean );
    procedure ReadData( Reader :TReader );
    procedure WriteData( Writer :TWriter );
  protected
    procedure DefineProperties( Filer :TFiler ); override;
  public
    constructor Create( anOwner :TComponent ); override;
    destructor Destroy; override;
    function Execute :Boolean;
    property NameMappings :Pointer
      read FData.hNameMappings;
    property OperationAborted :Bool
      read FData.fAnyOperationsAborted;
    property OperandFrom :String
      write SetFrom;
    property OperandTo :String
      write SetTo;
  published
    property Operation :TFileOperation
      read GetOperation write SetOperation stored false;
    property Flags :TFileOperationFlags
      read GetFlags  write SetFlags stored false;
    property ProgressTitle :String
      read FProgressTitle write FProgressTitle;
    property WantMappingHandle :Boolean
      read GetWantMappingHandle write SetWantMappingHandle stored false;
    property ParentForm :TForm
      read FParentForm write FParentForm;
  end;

implementation


{ TFileOperator }

procedure TFileOperator.SetOperation( Value :TFileOperation );
begin
 with FData do
  case Value of
    foCopy : wFunc := FO_COPY;
    foDelete : wFunc := FO_DELETE;
    foRename : wFunc := FO_RENAME;
    foMove : wFunc := FO_MOVE;
  end;
end;

function TFileOperator.GetOperation :TFileOperation;
begin
  result := foCopy;
  case FData.wFunc of
    FO_COPY : result := foCopy;
    FO_DELETE : result := foDelete;
    FO_RENAME : result := foRename;
    FO_MOVE : result := foMove;
  end;
end;

procedure TFileOperator.SetFrom( const Value :String );
var I, L :Integer;
begin
  FFrom := Value;
  L := Length( Value );
  for i := 1 to L do if FFrom[ i ] = ',' then FFrom[ i ] := #0;
  SetLength( FFrom, L + 1 );
  FFrom[ L + 1 ] := #0;
end;

procedure TFileOperator.SetTo( const Value :String );
var I, L :Integer;
begin
  FTo := Value;
  L := Length( Value );
  for i := 1 to L do if FTo[ i ] = ',' then FTo[ i ] := #0;
  SetLength( FTo, L + 1 );
  FTo[ L + 1 ] := #0;
end;

function TFileOperator.GetWantMappingHandle :Boolean;
begin
 result := GetOperFlag( FOF_WANTMAPPINGHANDLE );
end;

procedure TFileOperator.SetWantMappingHandle ( Value :Boolean );
begin
 SetOperFlag( FOF_WANTMAPPINGHANDLE, Value );
end;

procedure TFileOperator.SetFlags( Value :TFileOperationFlags );
begin
  SetOperFlag( FOF_ALLOWUNDO, foAllowUndo in Value );
  SetOperFlag( FOF_CONFIRMMOUSE, foConfirmMouse in Value );
  SetOperFlag( FOF_FILESONLY, foFilesOnly in Value );
  SetOperFlag( FOF_MULTIDESTFILES, foMultiDestFiles in Value );
  SetOperFlag( FOF_NOCONFIRMATION, foNoConfirmation in Value );
  SetOperFlag( FOF_NOCONFIRMMKDIR, foNoConfirmMkDir in Value );
  SetOperFlag( FOF_RENAMEONCOLLISION, foRenameOnCollision in Value );
  SetOperFlag( FOF_SILENT, foSilent in Value );
  SetOperFlag( FOF_SIMPLEPROGRESS, foSimpleProgress in Value );
end;

function TFileOperator.GetFlags :TFileOperationFlags;
begin
  result := [];
  if GetOperFlag( FOF_ALLOWUNDO ) then include( result, foAllowUndo );
  if GetOperFlag( FOF_CONFIRMMOUSE ) then include( result, foConfirmMouse );
  if GetOperFlag( FOF_FILESONLY ) then include( result, foFilesOnly );
  if GetOperFlag( FOF_MULTIDESTFILES ) then include( result, foMultiDestFiles );
  if GetOperFlag( FOF_NOCONFIRMATION ) then include( result, foNoConfirmation );
  if GetOperFlag( FOF_NOCONFIRMMKDIR ) then include( result, foNoConfirmMkDir );
  if GetOperFlag( FOF_RENAMEONCOLLISION ) then include( result, foRenameOnCollision );
  if GetOperFlag( FOF_SILENT ) then include( result, foSilent );
  if GetOperFlag( FOF_SIMPLEPROGRESS ) then include( result, foSimpleProgress );
end;

function TFileOperator.GetOperFlag( F :Cardinal ):boolean;
begin
  result := ( FData.fFlags and F ) <> 0;
end;

procedure TFileOperator.SetOperFlag( F :Cardinal; V :Boolean );
begin
 with FData do
  if V then fFlags := fFlags or F else fFlags := fFlags and ( not F );
end;

procedure TFileOperator.DefineProperties( Filer :TFiler );
begin
 Inherited DefineProperties( Filer );
 Filer.DefineProperty( 'data', ReadData, WriteData, true );
end;

procedure TFileOperator.ReadData( Reader :TReader );
begin
  Reader.Read( FData, SizeOf( FData ) );
end;

procedure TFileOperator.WriteData( Writer :TWriter );
begin
  writer.write( FData, SizeOf( FData ) );
end;

constructor TFileOperator.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FData.fFlags := FOF_ALLOWUNDO OR FOF_NOCONFIRMMKDIR;
 FProgressTitle := 'File  Operation';
end;

function TFileOperator.Execute :Boolean;
begin
 FData.pFrom := PChar( FFrom );
 FData.pTo := PChar( FTo );
 if FParentForm <> nil
  then FData.Wnd := FParentForm.Handle
  else FData.Wnd := 0;
 result := ShFileOperation( FData ) = 0;
end;

destructor TFileOperator.Destroy;
begin
  SetLength( FFrom, 0 );
  SetLength( FTo, 0 );
  inherited Destroy;
end;



end.
